@@ -25,6 +25,7 @@
 #include "ct_export2html.h"
 #include "ct_pref_dlg.h"
 #include "ct_clipboard.h"
+#include "ct_storage_control.h"
 #include <gtkmm/dialog.h>
 #include <gtkmm/stock.h>
 #include <cstdlib>
@@ -282,7 +283,7 @@ void CtActions::link_clicked(const Glib::ustring& tag_property_value, bool from_
     }
     else if (link_entry.type == CtConst::LINK_TYPE_FILE) // link to file
     {
-        fs::path filepath = CtExport2Html::_link_process_filepath(link_entry.file);
+        fs::path filepath = CtExport2Html::_link_process_filepath(link_entry.file, _pCtMainWin->get_ct_storage()->get_file_path().parent_path().string());
         if (not Glib::file_test(filepath.string(), Glib::FILE_TEST_IS_REGULAR))
         {
             CtDialogs::error_dialog(str::format(_("The File Link '%s' is Not Valid"), filepath.string()), *_pCtMainWin);
@@ -294,7 +295,7 @@ void CtActions::link_clicked(const Glib::ustring& tag_property_value, bool from_
     }
     else if (link_entry.type == CtConst::LINK_TYPE_FOLD) // link to folder
     {
-        fs::path folderpath = CtExport2Html::_link_process_folderpath(link_entry.fold).c_str();
+        fs::path folderpath = CtExport2Html::_link_process_folderpath(link_entry.fold, _pCtMainWin->get_ct_storage()->get_file_path().parent_path().string()).c_str();
         if (not fs::is_directory(folderpath))
         {
             CtDialogs::error_dialog(str::format(_("The Folder Link '%s' is Not Valid"), folderpath.string()), *_pCtMainWin);
@@ -404,7 +405,7 @@ void CtActions::exec_code()
 {
     if (!_is_there_selected_node_or_error()) return;
 
-    Glib::ustring code_type;
+    std::string code_type;
     Glib::ustring code_val;
     auto proof = _get_text_view_n_buffer_codebox_proof();
     if (proof.from_codebox) {
@@ -418,24 +419,12 @@ void CtActions::exec_code()
         code_type = _pCtMainWin->curr_tree_iter().get_node_syntax_highlighting();
         code_val = _curr_buffer()->begin().get_text(_curr_buffer()->end());
     }
-    std::string binary_cmd = [&]() -> std::string {
-        for (auto& it: _pCtMainWin->get_ct_config()->customCodexecType)
-            if (it.first == code_type) return it.second;
-        for (const auto& it: CtConst::CODE_EXEC_TYPE_CMD_DEFAULT)
-            if (it.first == code_type) return it.second;
-        return "";
-    }();
+    std::string binary_cmd = CtPrefDlg::get_code_exec_type_cmd(_pCtMainWin, code_type);
     if (binary_cmd.empty()) {
         CtDialogs::warning_dialog(str::format(_("You must associate a command to '%s'.\nDo so in the Preferences Dialog"), code_type), *_pCtMainWin);
         return;
     }
-    std::string code_type_ext = [&]() -> std::string {
-        for (auto& it: _pCtMainWin->get_ct_config()->customCodexecExt)
-            if (it.first == code_type) return it.second;
-        for (const auto& it: CtConst::CODE_EXEC_TYPE_EXT_DEFAULT)
-            if (it.first == code_type) return it.second;
-        return "text";
-    }();
+    std::string code_type_ext = CtPrefDlg::get_code_exec_ext(_pCtMainWin, code_type);
     Glib::ustring code_exec_term = CtPrefDlg::get_code_exec_term_run(_pCtMainWin);
 
     fs::path filepath_src_tmp = _pCtMainWin->get_ct_tmp()->getHiddenFilePath("exec_code." + code_type_ext);
@@ -452,7 +441,7 @@ void CtActions::exec_code()
     // if std::system is not enougth, then try g_spawn_async_with_pipes
     int status = std::system(terminal_cmd.c_str());
 
-// todo: fix win32
+
 #ifdef _WIN32
 #define WEXITSTATUS(x) x
 #endif
@@ -500,41 +489,57 @@ void CtActions::codebox_save_to_file()
 void CtActions::codebox_increase_width()
 {
     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
-    if (curr_codebox_anchor->get_width_in_pixels())
-         curr_codebox_anchor->set_width_height(curr_codebox_anchor->get_frame_width() + CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX, 0);
-     else
-         curr_codebox_anchor->set_width_height(curr_codebox_anchor->get_frame_width() + CtCodebox::CB_WIDTH_HEIGHT_STEP_PERC, 0);
+    int prevFrameWidth = curr_codebox_anchor->get_frame_width();
+    if (_pCtMainWin->get_ct_config()->codeboxAutoResize and prevFrameWidth < curr_codebox_anchor->get_text_view().get_allocated_width() ) {
+        prevFrameWidth = curr_codebox_anchor->get_text_view().get_allocated_width();
+    }
+    if (curr_codebox_anchor->get_width_in_pixels()) {
+        curr_codebox_anchor->set_width_height(prevFrameWidth + CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX, 0);
+    }
+    else {
+        curr_codebox_anchor->set_width_height(prevFrameWidth + CtCodebox::CB_WIDTH_HEIGHT_STEP_PERC, 0);
+    }
+    _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf);
 }
 
 // Decrease CodeBox Width
 void CtActions::codebox_decrease_width()
 {
-     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
-     if (curr_codebox_anchor->get_width_in_pixels())
-     {
-         if (curr_codebox_anchor->get_frame_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX >= CtCodebox::CB_WIDTH_LIMIT_MIN)
-             curr_codebox_anchor->set_width_height(curr_codebox_anchor->get_frame_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX, 0);
-     }
-     else
-     {
-         if (curr_codebox_anchor->get_frame_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PERC >= CtCodebox::CB_WIDTH_LIMIT_MIN)
-             curr_codebox_anchor->set_width_height(curr_codebox_anchor->get_frame_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PERC, 0);
-     }
+    if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
+    if (curr_codebox_anchor->get_width_in_pixels()) {
+        if (curr_codebox_anchor->get_frame_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX >= CtCodebox::CB_WIDTH_LIMIT_MIN) {
+            curr_codebox_anchor->set_width_height(curr_codebox_anchor->get_frame_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX, 0);
+            _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf);
+        }
+    }
+    else {
+        if (curr_codebox_anchor->get_frame_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PERC >= CtCodebox::CB_WIDTH_LIMIT_MIN) {
+            curr_codebox_anchor->set_width_height(curr_codebox_anchor->get_frame_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PERC, 0);
+            _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf);
+        }
+    }
 }
 
 // Increase CodeBox Height
 void CtActions::codebox_increase_height()
 {
-     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
-     curr_codebox_anchor->set_width_height(0, curr_codebox_anchor->get_frame_height() + CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX);
+    if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
+    int prevFrameHeight = curr_codebox_anchor->get_frame_height();
+    if (_pCtMainWin->get_ct_config()->codeboxAutoResize and prevFrameHeight < curr_codebox_anchor->get_text_view().get_allocated_height() ) {
+        prevFrameHeight = curr_codebox_anchor->get_text_view().get_allocated_height();
+    }
+    curr_codebox_anchor->set_width_height(0, prevFrameHeight + CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX);
+    _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf);
 }
 
 // Decrease CodeBox Height
 void CtActions::codebox_decrease_height()
 {
-     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
-     if (curr_codebox_anchor->get_frame_height() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX >= CtCodebox::CB_HEIGHT_LIMIT_MIN)
-         curr_codebox_anchor->set_width_height(0, curr_codebox_anchor->get_frame_height() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX);
+    if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
+    if (curr_codebox_anchor->get_frame_height() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX >= CtCodebox::CB_HEIGHT_LIMIT_MIN) {
+        curr_codebox_anchor->set_width_height(0, curr_codebox_anchor->get_frame_height() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX);
+        _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf);
+    }
 }
 
 void CtActions::table_cut()
@@ -658,7 +663,7 @@ void CtActions::table_edit_properties()
 {
     if (!_is_curr_node_not_read_only_or_error()) return;
     _pCtMainWin->get_ct_config()->tableColWidth = curr_table_anchor->get_col_width();
-    if (!_table_dialog(_("Edit Table Properties"), false))
+    if (CtDialogs::TableHandleResp::Cancel == CtDialogs::table_handle_dialog(_pCtMainWin, _("Edit Table Properties"), false/*is_insert*/))
         return;
     curr_table_anchor->set_col_width(_pCtMainWin->get_ct_config()->tableColWidth);
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true /*new_machine_state*/);
@@ -672,7 +677,6 @@ void CtActions::table_export()
     args.filter_name=_("CSV File");
     args.filter_pattern={"*.csv"};
 
-
     Glib::ustring filename = CtDialogs::file_save_as_dialog(args);
     if (filename.empty()) return;
     if (!str::endswith(filename, ".csv")) filename += ".csv";
