@@ -43,6 +43,15 @@ CtConfig::~CtConfig()
 
 bool CtConfig::load_from_file(const fs::path& filepath)
 {
+#ifdef _WIN32
+    // compatibility with old pygtk2 version
+    if (not fs::exists(filepath)) {
+        const std::string old_pygtk2_filepath = str::replace(filepath.string(), "\\Local\\", "\\Roaming\\");
+        if (fs::exists(old_pygtk2_filepath)) {
+            fs::copy_file(old_pygtk2_filepath, filepath);
+        }
+    }
+#endif // _WIN32
     if (fs::exists(filepath))
     {
         _uKeyFile = std::make_unique<Glib::KeyFile>();
@@ -230,13 +239,17 @@ void CtConfig::_populate_keyfile_from_data()
     _uKeyFile->set_boolean(_currentGroup, "auto_smart_quotes", autoSmartQuotes);
     _uKeyFile->set_boolean(_currentGroup, "triple_click_paragraph", tripleClickParagraph);
     _uKeyFile->set_boolean(_currentGroup, "enable_symbol_autoreplace", enableSymbolAutoreplace);
+#ifdef MD_AUTO_REPLACEMENT
     _uKeyFile->set_boolean(_currentGroup, "enable_md_formatting", enableMdFormatting);
+#endif // MD_AUTO_REPLACEMENT
     _uKeyFile->set_integer(_currentGroup, "wrapping_indent", wrappingIndent);
     _uKeyFile->set_boolean(_currentGroup, "auto_indent", autoIndent);
     _uKeyFile->set_boolean(_currentGroup, "rt_show_white_spaces", rtShowWhiteSpaces);
     _uKeyFile->set_boolean(_currentGroup, "pt_show_white_spaces", ptShowWhiteSpaces);
     _uKeyFile->set_boolean(_currentGroup, "rt_highl_curr_line", rtHighlCurrLine);
     _uKeyFile->set_boolean(_currentGroup, "pt_highl_curr_line", ptHighlCurrLine);
+    _uKeyFile->set_boolean(_currentGroup, "rt_highl_match_bra", rtHighlMatchBra);
+    _uKeyFile->set_boolean(_currentGroup, "pt_highl_match_bra", ptHighlMatchBra);
     _uKeyFile->set_integer(_currentGroup, "space_around_lines", spaceAroundLines);
     _uKeyFile->set_integer(_currentGroup, "relative_wrapped_space", relativeWrappedSpace);
     _uKeyFile->set_string(_currentGroup, "h_rule", hRule);
@@ -488,30 +501,42 @@ void CtConfig::_populate_data_from_keyfile()
     _populate_bool_from_keyfile("auto_smart_quotes", &autoSmartQuotes);
     _populate_bool_from_keyfile("triple_click_paragraph", &tripleClickParagraph);
     _populate_bool_from_keyfile("enable_symbol_autoreplace", &enableSymbolAutoreplace);
+#ifdef MD_AUTO_REPLACEMENT
     _populate_bool_from_keyfile("enable_md_formatting", &enableMdFormatting);
+#endif // MD_AUTO_REPLACEMENT
     _populate_int_from_keyfile("wrapping_indent", &wrappingIndent);
     _populate_bool_from_keyfile("auto_indent", &autoIndent);
     _populate_bool_from_keyfile("rt_show_white_spaces", &rtShowWhiteSpaces);
     _populate_bool_from_keyfile("pt_show_white_spaces", &ptShowWhiteSpaces);
     _populate_bool_from_keyfile("rt_highl_curr_line", &rtHighlCurrLine);
     _populate_bool_from_keyfile("pt_highl_curr_line", &ptHighlCurrLine);
+    _populate_bool_from_keyfile("rt_highl_match_bra", &rtHighlMatchBra);
+    _populate_bool_from_keyfile("pt_highl_match_bra", &ptHighlMatchBra);
     _populate_int_from_keyfile("space_around_lines", &spaceAroundLines);
     _populate_int_from_keyfile("relative_wrapped_space", &relativeWrappedSpace);
     _populate_string_from_keyfile("h_rule", &hRule);
     _populate_string_from_keyfile("special_chars", &specialChars);
     _populate_string_from_keyfile("selword_chars", &selwordChars);
     _populate_string_from_keyfile("chars_listbul", &charsListbul);
+    if (charsListbul.size() < CtConst::CHARS_LISTBUL_DEFAULT.size()) {
+        charsListbul = CtConst::CHARS_LISTBUL_DEFAULT;
+    }
     _populate_string_from_keyfile("chars_toc", &charsToc);
-
-    // bug fix for alfa version when charsTodo is really long
-    Glib::ustring tempCharsToDo;
-    _populate_string_from_keyfile("chars_todo", &tempCharsToDo);
-    if (tempCharsToDo.size() > 4 && tempCharsToDo[0] == tempCharsToDo[3])
-        tempCharsToDo = CtConst::CHARS_TODO_DEFAULT;
-    charsTodo = tempCharsToDo;
-
+    if (charsToc.size() < CtConst::CHARS_TOC_DEFAULT.size()) {
+        charsToc = CtConst::CHARS_TOC_DEFAULT;
+    }
+    _populate_string_from_keyfile("chars_todo", &charsTodo);
+    if (charsTodo.size() != CtConst::CHARS_TODO_DEFAULT.size()) {
+        charsTodo = CtConst::CHARS_TODO_DEFAULT;
+    }
     _populate_string_from_keyfile("chars_smart_dquote", &chars_smart_dquote);
+    if (chars_smart_dquote.size() != CtConst::CHARS_SMART_DQUOTE_DEFAULT.size()) {
+        chars_smart_dquote = CtConst::CHARS_SMART_DQUOTE_DEFAULT;
+    }
     _populate_string_from_keyfile("chars_smart_squote", &chars_smart_squote);
+    if (chars_smart_squote.size() != CtConst::CHARS_SMART_SQUOTE_DEFAULT.size()) {
+        chars_smart_squote = CtConst::CHARS_SMART_SQUOTE_DEFAULT;
+    }
     _populate_string_from_keyfile("latest_tag_prop", &latestTagProp);
     _populate_string_from_keyfile("latest_tag_val", &latestTagVal);
     _populate_string_from_keyfile("timestamp_format", &timestampFormat);
